
input.jq-couponcode-part {
  font-family: monospace;
  font-size: 22px;
  width: 60px;
  text-align: left;
  padding: 2px 23px 0px 3px;
}

input.jq-couponcode-good {
  background-color: #77ff77;
  background-image: url(cc_icons.png);
  background-position: 52px -56px;
}

input.jq-couponcode-good-nohighlight {
  text-align: center;
  padding: 2px 10px 2px 10px;
}

input.jq-couponcode-bad {
  background-color: #ff7777;
  background-image: url(cc_icons.png);
  background-position: 52px -103px;
}

span.jq-couponcode-sep {
  font-family: monospace;
  font-weight: bold;
  padding: 0 0.2em;
}

